function [RTau]=GETAUTOCOV(k,theta,tau)
% Simulation method to find the autocovariance function of a Alternating Renewal Gamma process.
% copyright: hung.nguyen@adelaide.edu.au

% Simulate the process
clear A_rseq R_rseq ON_rseq OFF_rseq rsamples;

timeInt=0.1;
n_rsamples=max(floor(50*(k(1)*theta(1)+k(2)*theta(2))/timeInt),floor(100*tau(length(tau)))); %% At least 1000 ON/OFF periods
timeMax=floor(n_rsamples*timeInt)+1;

n_rsamples


%%%%%% Generate the Gamma Alternating ON-OFF process
A_rseq = [0:timeInt:(timeMax+timeInt)];
ON_rseq= mygamrnd(k(1),theta(1),2*n_rsamples,0);
OFF_rseq= mygamrnd(k(2),theta(2),2*n_rsamples,0);

R_rseq=[0];

idx=1;
i=1;
while ((R_rseq(idx)<=A_rseq(length(A_rseq))) && ( i<= length(ON_rseq)))
    R_rseq(idx+1) =R_rseq(idx) + ON_rseq(i);
    R_rseq(idx+2)= R_rseq(idx+1) + OFF_rseq(i);
    idx=idx+2;
    i=i+1;
end


R_rseq

%%%% Sampling the ON-OFF process
current_j=1;
for i=1:n_rsamples
    for j=current_j:length(R_rseq)-1
        if ((A_rseq(i) > R_rseq(j))&&(A_rseq(i)<=R_rseq(j+1)))
            current_j=j;
            break;
        end
    end
    
    if (mod(current_j,2) == 1)
        rsamples(i)=1;
    else
        rsamples(i)=0;
    end
    
end

% Using simulation
V_SIM=0;
idx=1;
RTau=[];

for tidx=1:length(tau)  
    %%%% Computing the Autocovariance function
    step=floor(tau(tidx)/timeInt);
    tem=cov(rsamples(1:n_rsamples -step),rsamples([1:n_rsamples-step]+step));
    RTau(idx)=tem(1,2);%Eij/count-mean(rsamples)^2;
    idx=idx+1;
    
end


