function [ACF,stderr,lpq,qpval,accept_hypo,lag] = bl(Z,N);
%  Calculates autocorrelations for multiple data series.
%  Missing values in Z (NaN) are considered.
%  Also calculates Ljung-Box Q stats and p-values.
%
%    [AutoCorr,stderr,lpq,qpval] = acorf(Z,N);
%  If mean should be removed use
%    [AutoCorr,stderr,lpq,qpval] = acorf(detrend(Z',0)',N);
%  If trend should be removed use
%    [AutoCorr,stderr,lpq,qpval] = acorf(detrend(Z')',N);
%
% INPUT
%  Z	is data series for which autocorrelations are required
%       each in a row
%  N	maximum lag
%
% OUTPUT
%  AutoCorr nr x N matrix of autocorrelations
%  stderr   nr x N matrix of (approx) std errors
%  lpq      nr x M matrix of Ljung-Box Q stats
%  qpval    nr x N matrix of p-values on Q stats
%
% All input and output parameters are organized in rows, one row
% corresponds to one series
%
% REFERENCES:
% calculating lpq, stderr, qpval from
% suggested by Philip Gray, University of New South Wales,

[nr,nc] = size(Z);
NC = sum(~isnan(Z),2); % missing values

%AUTOCOV = acovf(Z,N);
%AUTOCOV = AUTOCOV(:,2:N+1) ./ AUTOCOV(:,ones(1,N));

[ACF b]=autocorr(Z.',0:1:N);
ACF = ACF(2:N+1);
%if nargout > 1
stderr = sqrt(1./NC)*ones(1,N);
lpq = zeros(nr,N);
qpval = zeros(nr,N);

cum=zeros(nr,1);
for k=1:N,
    cum = cum+ACF(k).*conj(ACF(k))./(NC-k);
    lpq(:,k) = NC.*(NC+2).*cum;                 % Ljung box Q for k lags
    qpval(:,k) = 1 - chi2cdf(lpq(:,k),k);	% p-value of Q stat

end;

chi_table = load('chi_square_table.txt');
chi_value = chi_table(N,2);

lag=0;

for k=1:N
    if (lpq(1,k) > chi_table(k,2))
        lag =k;
        break;
    end
end

%test for the maximum lag provided
pval = qpval(1,N);
accept_hypo = (lpq(1,N) < chi_value);

