function [renewal] = isRenewal(trce)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Renewal test: test to see if the trace fits a renewal ON-OFF model
%%%% PREREQUISITE:
%%%%       (1) Test the trace for stationarity using stationaryCheck.m
%%%% INPUT:
%%%%       (1) The trace
%%%%       (2) The crosscorrThres: if there is significant correlation
%%%% OUTPUT: isRenewal =1 if the trace is renewal, 0 otherwise
%%%% COPYRIGHT : Hung X. Nguyen, the University of Adelaide, Australia
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% Parameters used in the tests
crossCorrThres =0.7; %%% if the cross correlation is greater than this threshold
%%% the two timeseries are correlated


%%%% Variables used to stored the output
corrLNL=[];

lagSamplebl  =[];
lagArraybl =[];

lossArray=[];
lagL=[];
lagNL=[];

lagLbl=[];
lagNLbl=[];

isRenewal=1;


%%% Computing Loss and Good Run Length
L=[];
NL=[];
SL=[];

i=1;

LR =0;
NLR=0;
while (i <= length(trce))
    if ((trce(i)==1)&(NLR>0))
        NLR=NLR+1;
    elseif  ((trce(i)==1)&(NLR==0))
        if (LR >0)
            L=[L LR];
        end
        LR=0;
        NLR=NLR+1;
    elseif ((trce(i)==0)&(LR>0))
        LR =LR+1;
    elseif ((trce(i)==0)&(LR==0))
        if (NLR>0)
            NL=[NL NLR];
        end
        NLR=0;
        LR=LR+1;
    end
    i=i+1;
end


if (length(L)<length(NL))
    NL=NL(1:length(L));
elseif (length(L)>length(NL))
    L=L(1:length(NL));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Testing for independence in the loss runs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

laL=-100;
if (length(L)>10)
    %%% Testing for independence using the ACF test
    [c boundL]=autocorr(L);
    for i=1:length(c)
        if abs(c(i))<boundL
            laL=i;
            break
        end
    end
    
    
    if laL >0
        laL=laL-2;
    end
    
    lagL =[lagL laL];
    
    
    %%% Testing for independence using the bl test
    [ACF,stderr,lpq,qpval,accept_hypotrce,laLbl] = bl(L,min(50,length(L)-2));
    lagLbl=[lagLbl laLbl];
    
    isRenewal=min(isRenewal,accept_hypotrce);
    
else
    lagL =[lagL -10];
    lagLbl=[lagLbl -10];
    isRenewal=0;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Testing for independence of the good runs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

laNL=-100;
if (length(NL)>10)
    
    %%% Computing the correlation time-scale using the ACF test
    [c boundNL]=autocorr(NL);
    for i=1:length(c)
        if abs(c(i))<boundNL
            laNL=i;
            break
        end
    end
    
    if laNL >0
        laNL=laNL-2;
    end
    
    lagNL =[lagNL laNL];
    
    %%% Testing for independence using the bl test
    [ACF,stderr,lpq,qpval,accept_hypotrce,laNLbl] = bl(NL,min(50,length(L)-2));
    lagNLbl=[lagNLbl laNLbl];
    
    isRenewal = min(isRenewal,accept_hypotrce);
else
    
    lagNL =[lagNL -10];
    lagNLbl=[lagNLbl -10];
    
    isRenewal=0;
end

%%%%%%% Correlation between good and loss run
if (length(NL)>10)
    tempCorr = corrcoef(L,NL);
    corrLNL=[corrLNL  tempCorr(1,2)];   
    
    if (tempCorr(1,2) >crossCorrThres)
        isRenewal=0;
    end
end
renewal=isRenewal;



