
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Infer the parameters of the Gamma Alternating Renewal process
% from sampling data
% copyright: hung.nguyen@adelaide.edu.au
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
ONP=1/5; %%%% 1/ average length of the Lossy period
OFFP =1/100; %%%1/average length of the Lossless period
noRep=1;

kArray=[];
thetaArray=[];

%%%gamma dist
ko=[4 5];
thetao=[1/(ko(1)*ONP)  1/(ko(2)*OFFP)];
lambda = 0.01; %%% Sampling rate

lambdaArray=[0.01:0.02:0.1,0.1:0.2:1];
k1Array=[1:1:10];
k2Array=[1:1:10];

for lambdaID =1:1:10
    lambda=lambdaArray(lambdaID);
    %theta(2)=thetaidx;
    for k1ID=2:1:2
        ko(1)=k1Array(k1ID);
        for k2ID=5:1:5
            ko(2)=k2Array(k2ID);
            
            for rep=1:noRep
                
                thetao=[1/(ko(1)*ONP)  1/(ko(2)*OFFP)];
                n_samples=min(20000,floor(30*(1/OFFP +1/ONP)/lambda));
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
                disp(['Simulating the Alternating Renewal Gamma Process with parameters:',...
                    ' lambda=',num2str(lambda),' ,k(1)=',num2str(ko(1)),' ,k(2)=',...
                     num2str(ko(2)),' ,theta(1)=',num2str(thetao(1)),' ,theta(2)=',num2str(thetao(2))]);
                 
                
                %%%%%% Gamma
                trce=-1*ones(n_samples,1);
                A_seq = cumsum(- log(rand(n_samples,1))/lambda); %% arrival time of samples;
                ON_seq= mygamrnd(ko(1),thetao(1),2*n_samples,0);
                OFF_seq= mygamrnd(ko(2),thetao(2),2*n_samples,0);
                
                R_seq=[0]; %%% Continuous-time loss process
                
                idx=1;
                for i=1:2*n_samples
                    R_seq(idx+1) =R_seq(idx) + ON_seq(i);
                    R_seq(idx+2)= R_seq(idx+1) + OFF_seq(i);
                    idx=idx+2;
                end
                
                %%%% Simulating sampling process
                current_j=1;
                for i=1:n_samples
                    found =0;
                    for j=current_j:length(R_seq)-1
                        if ((A_seq(i) > R_seq(j))&&(A_seq(i)<=R_seq(j+1)))
                            current_j=j;
                            found=1;
                            break;
                        end
                    end
                    if found ==1
                        if (mod(current_j,2) == 1)
                            trce(i)=0; %%%loss
                        else
                            trce(i)=1; %%% no-loss
                        end
                    end
                    
                end
                
                trceTime=A_seq;
                timeInt=1;
                
                
                %%%% Computing observation array for input into the hsmm algorithm
                maxTime=trceTime(length(trceTime))-trceTime(1);
                timeArray = trceTime(1) + [0:timeInt:(maxTime+timeInt)];
                
                binArray=(2)*ones(1,length(timeArray)); %%% 2 is miss, 0 is loss, 1 is good
                for t=2:length(timeArray)
                    tempk=find((trceTime>=timeArray(t-1))&(trceTime<timeArray(t)));
                    if length(tempk)>0
                        binArray(t) = trce(tempk(1));
                    end
                end
                
                
                %++++++++ The hidden semi-Markov model +++++++++++
                
                % O(1:T)          The observation sequence
                % D               The maximum duration of states
                % K               The total number of observation values
                % PAI(1:M,1:1)    the initial probability of states
                %
                % A(1:M,1:M)      The state Transition Probability Matrix
                %
                % P(1:M,1:D);      The probability of state duration
                %
                % B(1:M,1:K)       The observation probability distribution
                
                
                
                O = binArray+1;
                PAI=[0.1; 0.9];
                A = ones(2,2);
                for r=1:2
                    A(r,r)=0;
                end
                
                D=4000;
                P=zeros(2,D);
                for i=1:2
                    for j=1:D
                        P(i,j) =  distF(thetao(i),ko(i),j,lambda);
                    end
                end
                B=[1-exp(-lambda) 0 exp(-lambda);0 1-exp(-lambda) exp(-lambda)]; %%% 3 is unobsrved; 1 is loss, 2 is good
                IterationNo=60;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % Calling SAIL
                disp('Calling SAIL ...');
                tic
                [PAI,A,B,P_est,Qest,lkh] = hsmm_new(PAI,A,B,P,O,IterationNo);
                toc
                
                meanEst=[0 0];
                meanS=[0 0];
                varianceEst=[0 0];
                for i=1:2
                    for j=1:D
                        meanEst(i) =meanEst(i)+(j)*P_est(i,j);
                        meanS(i)=meanS(i)+(j)^2*P_est(i,j);
                        
                    end
                    variance(i)= meanS(i) - meanEst(i)*meanEst(i);
                end
                theta =variance./meanEst;
                k=meanEst./theta;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%% Collecting statistics
                disp('Collecting statistics ...');
                kArray=[kArray;k];
                thetaArray=[thetaArray;theta];
            end
        end
    end
end
disp('k(1) k(2)');
disp(kArray);
disp('theta(1) theta(2)');
disp(thetaArray);    
